@@ -149,7 +149,7 @@ bool title_matches(ExpressionItem *item, const string &str, size_t minlength) {
 	while(true) {
 		while(true) {
 			if(i >= title.length()) return false;
-			if(title[i] != ' ') break;
+			if(title[i] != ' ' && title[i] != '(') break;
 			i++;
 		}
 		size_t i2 = title.find(' ', i);
@@ -163,36 +163,67 @@ bool title_matches(ExpressionItem *item, const string &str, size_t minlength) {
 	}
 	return false;
 }
+bool test_unicode_length_from(const string &str, size_t i, size_t l) {
+	if(l == 0) return true;
+	for(; i < str.length(); i++) {
+		if((signed char) str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
+			l--;
+			if(l == 0) return true;
+		}
+	}
+	return false;
+}
 bool name_matches(ExpressionItem *item, const string &str) {
 	for(size_t i2 = 1; i2 <= item->countNames(); i2++) {
-		if(item->getName(i2).case_sensitive) {
-			if(str == item->getName(i2).name.substr(0, str.length())) {
+		const ExpressionName *ename = &item->getName(i2);
+		if(ename->case_sensitive) {
+			if(str == ename->name.substr(0, str.length())) {
 				return true;
 			}
 		} else {
-			if(equalsIgnoreCase(str, item->getName(i2).name, 0, str.length(), 0)) {
+			if(equalsIgnoreCase(str, ename->name, 0, str.length(), 0)) {
 				return true;
 			}
-			unordered_map<const ExpressionName*, string>::iterator cap_it = capitalized_names.find(&item->getName(i2));
+			unordered_map<const ExpressionName*, string>::iterator cap_it = capitalized_names.find(ename);
 			if(cap_it != capitalized_names.end() && equalsIgnoreCase(str, cap_it->second, 0, str.length(), 0)) {
 				return true;
 			}
 		}
+		size_t i = 0;
+		while(true) {
+			i = ename->name.find("_", i);
+			if(i == string::npos || !test_unicode_length_from(ename->name, i + 1, 2)) break;
+			i++;
+			if((ename->case_sensitive && str == ename->name.substr(i, str.length())) || (!ename->case_sensitive && equalsIgnoreCase(str, ename->name, i, str.length() + i, 0))) {
+				return true;
+			}
+		}
 	}
 	return false;
 }
 int name_matches2(ExpressionItem *item, const string &str, size_t minlength, size_t *i_match = NULL) {
-	if(minlength > 1 && unicode_length(str) == 1) return 0;
+	if(minlength > 1 && !test_unicode_length_from(str, 0, 2)) return 0;
 	bool b_match = false;
 	for(size_t i2 = 1; i2 <= item->countNames(); i2++) {
-		if(equalsIgnoreCase(str, item->getName(i2).name, 0, str.length(), 0)) {
-			if(!item->getName(i2).case_sensitive && item->getName(i2).name.length() == str.length()) {
+		const ExpressionName *ename = &item->getName(i2);
+		if(equalsIgnoreCase(str, ename->name, 0, str.length(), 0)) {
+			if(!ename->case_sensitive && ename->name.length() == str.length()) {
 				if(i_match) *i_match = i2;
 				return 1;
 			}
 			if(i_match && *i_match == 0) *i_match = i2;
 			b_match = true;
 		}
+		size_t i = 0;
+		while(true) {
+			i = ename->name.find("_", i);
+			if(i == string::npos || !test_unicode_length_from(ename->name, i + 1, 2)) break;
+			i++;
+			if((ename->case_sensitive && str.length() <= ename->name.length() - i && str == ename->name.substr(i, str.length())) || (!ename->case_sensitive && equalsIgnoreCase(str, ename->name, i, str.length() + i, minlength))) {
+				b_match = true;
+				break;
+			}
+		}
 	}
 	return b_match ? 2 : 0;
 }
@@ -381,14 +412,19 @@ void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewCo
 	GtkTextIter object_start, object_end;
 	gtk_text_buffer_get_iter_at_offset(expression_edit_buffer(), &object_start, current_object_start);
 	gtk_text_buffer_get_iter_at_offset(expression_edit_buffer(), &object_end, current_object_end);
-	if(item && item->type() == TYPE_UNIT && ((Unit*) item)->subtype() == SUBTYPE_COMPOSITE_UNIT && (((CompositeUnit*) item)->countUnits() > 1 || !((CompositeUnit*) item)->get(1, &exp, &prefix) || exp != 1)) {
+	if(item && item->isHidden() && item->type() == TYPE_UNIT && item->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) item)->firstBaseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT && !((AliasUnit*) item)->firstBaseUnit()->isHidden() && ((AliasUnit*) item)->firstBaseUnit()->isActive()) {
+		PrintOptions po = printops;
+		po.can_display_unicode_string_arg = (void*) expression_edit_widget();
+		po.abbreviate_names = true;
+		str = ((AliasUnit*) item)->firstBaseUnit()->print(po, false, TAG_TYPE_HTML, true, false);
+	} else if(item && item->type() == TYPE_UNIT && item->subtype() == SUBTYPE_COMPOSITE_UNIT && (((CompositeUnit*) item)->countUnits() > 1 || !((CompositeUnit*) item)->get(1, &exp, &prefix) || exp != 1)) {
 		PrintOptions po = printops;
 		po.can_display_unicode_string_arg = (void*) expression_edit_widget();
 		po.abbreviate_names = true;
 		str = ((Unit*) item)->print(po, false, TAG_TYPE_HTML, true, false);
 	} else if(item) {
 		CompositeUnit *cu = NULL;
-		if(item->type() == TYPE_UNIT && ((Unit*) item)->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+		if(item->type() == TYPE_UNIT && item->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 			cu = (CompositeUnit*) item;
 			item = cu->get(1);
 		}
@@ -510,6 +546,7 @@ void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewCo
 								break;
 							}
 						}
+						if(b && i_match > 0 && i_type != 1 && equalsIgnoreCase(cap_it->second, gstr2)) b = false;
 					}
 					if(b) cap_str = cap_it->second;
 					if(!b) {
@@ -524,6 +561,7 @@ void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewCo
 								}
 							}
 						}
+						if(b && i_match > 0 && i_type != 1 && equalsIgnoreCase(ename->name, gstr2)) b = false;
 					}
 					if(!b) ename = NULL;
 				}
@@ -543,6 +581,7 @@ void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewCo
 								break;
 							}
 						}
+						if(ename && i_match > 0 && i_type != 1 && equalsIgnoreCase(ename->name, gstr2)) ename = NULL;
 					} else {
 						ename = NULL;
 					}
@@ -715,6 +754,9 @@ gboolean on_completionwindow_button_press_event(GtkWidget*, GdkEventButton*, gpo
 	return TRUE;
 }
 
+gboolean is_wayland(GdkDisplay* display) {
+	return strncmp(gdk_display_get_name(display), "wayland", strlen("wayland")) == 0;
+}
 void completion_resize_popup(int matches) {
 
 	gint x, y;
@@ -730,6 +772,7 @@ void completion_resize_popup(int matches) {
 	GtkRequisition tree_req;
 	GtkTreePath *path;
 	GtkTreeViewColumn *column;
+	gboolean are_coords_global;
 
 	GtkTextIter iter;
 	if(current_object_start < 0) {
@@ -769,6 +812,7 @@ void completion_resize_popup(int matches) {
 	GdkScreen *screen = gdk_display_get_default_screen(display);
 	gdk_screen_get_monitor_workarea(screen, gdk_screen_get_monitor_at_window(screen, window), &area);
 #endif
+	are_coords_global = !is_wayland(display);
 
 	items = matches;
 	if(items > 20) items = 20;
@@ -778,12 +822,14 @@ void completion_resize_popup(int matches) {
 		gtk_tree_path_free(path);
 		height = rect.y + rect.height - items_y + height_diff;
 	}
-	while(items > 0 && ((y > area.height / 2 && area.y + y < height) || (y <= area.height / 2 && area.height - y < height))) {
-		items--;
-		path = gtk_tree_path_new_from_indices(items - 1, -1);
-		gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
-		gtk_tree_path_free(path);
-		height = rect.y + rect.height - items_y + height_diff;
+	if(are_coords_global) {
+		while(items > 0 && ((y > area.height / 2 && area.y + y < height) || (y <= area.height / 2 && area.height - y < height))) {
+			items--;
+			path = gtk_tree_path_new_from_indices(items - 1, -1);
+			gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
+			gtk_tree_path_free(path);
+			height = rect.y + rect.height - items_y + height_diff;
+		}
 	}
 
 	gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(completion_scrolled), height);
@@ -795,25 +841,28 @@ void completion_resize_popup(int matches) {
 
 	if(popup_req.width < rect.width + 2) popup_req.width = rect.width + 2;
 
-	if(x < area.x) x = area.x;
-	else if(x + popup_req.width > area.x + area.width) x = area.x + area.width - popup_req.width;
+	if(are_coords_global) {
+		if(x < area.x) x = area.x;
+		else if(x + popup_req.width > area.x + area.width) x = area.x + area.width - popup_req.width;
 
-	if(y + bufloc.height + popup_req.height <= area.y + area.height || y - area.y < (area.y + area.height) - (y + bufloc.height)) {
-		y += bufloc.height;
+		if(y + bufloc.height + popup_req.height <= area.y + area.height || y - area.y < (area.y + area.height) - (y + bufloc.height)) {
+			y += bufloc.height;
+		} else {
+			path = gtk_tree_path_new_from_indices(matches - 1, -1);
+			gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
+			gtk_tree_path_free(path);
+			height = rect.y + rect.height + height_diff;
+			path = gtk_tree_path_new_from_indices(matches - items, -1);
+			gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
+			gtk_tree_path_free(path);
+			height -= rect.y;
+			gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(completion_scrolled), height);
+			gtk_widget_get_preferred_size(completion_window, &popup_req, NULL);
+			y -= popup_req.height;
+		}
 	} else {
-		path = gtk_tree_path_new_from_indices(matches - 1, -1);
-		gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
-		gtk_tree_path_free(path);
-		height = rect.y + rect.height + height_diff;
-		path = gtk_tree_path_new_from_indices(matches - items, -1);
-		gtk_tree_view_get_cell_area(GTK_TREE_VIEW(completion_view), path, column, &rect);
-		gtk_tree_path_free(path);
-		height -= rect.y;
-		gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(completion_scrolled), height);
-		gtk_widget_get_preferred_size(completion_window, &popup_req, NULL);
-		y -= popup_req.height;
+		y += bufloc.height;
 	}
-
 	if(matches > 0) {
 		path = gtk_tree_path_new_from_indices(0, -1);
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(completion_view), path, NULL, FALSE, 0.0, 0.0);
@@ -834,6 +883,17 @@ bool contains_related_unit(const MathStructure &m, Unit *u) {
 
 GtkTreeIter completion_separator_iter;
 extern bool display_expression_status;
+extern int to_form, to_base;
+
+const Number *from_struct_get_number(const MathStructure &m) {
+	if(m.isNumber()) return &m.number();
+	const Number *nr = NULL;
+	for(size_t i = 0; i < m.size(); i++) {
+		if(m[i].isNumber()) return &m[i].number();
+		else if(!nr) nr = from_struct_get_number(m[i]);
+	}
+	return nr;
+}
 
 void do_completion(bool to_menu) {
 	if(!enable_completion && !to_menu) {gtk_widget_hide(completion_window); return;}
@@ -847,9 +907,14 @@ void do_completion(bool to_menu) {
 		if(current_displayed_result()) u = find_exact_matching_unit(*current_displayed_result());
 		if(u) {
 			current_from_units.push_back(u);
-			if(u->subtype() == SUBTYPE_COMPOSITE_UNIT || !contains_prefix(current_displayed_result() ? *current_displayed_result() : *current_result())) {
+			if(u->subtype() == SUBTYPE_COMPOSITE_UNIT || !contains_prefix(*current_displayed_result())) {
 				exact_unit = u;
 			}
+			Unit *u2 = CALCULATOR->findMatchingUnit(*current_parsed_result());
+			int exp = 1;
+			if(u2 && u2 != u && u2->category() != u->category() && ((u2->baseUnit() == u->baseUnit() && u2->baseExponent() == u->baseExponent()) || (u->subtype() == SUBTYPE_COMPOSITE_UNIT && ((CompositeUnit*) u)->countUnits() == 1 && u2->baseUnit() == ((CompositeUnit*) u)->get(1, &exp)->baseUnit() && u2->baseExponent() == ((CompositeUnit*) u)->get(1)->baseExponent(exp)))) {
+				current_from_units.push_back(u2);
+			}
 		} else {
 			find_matching_units(*current_result(), current_parsed_result(), current_from_units);
 		}
@@ -866,7 +931,7 @@ void do_completion(bool to_menu) {
 	int to_type = 0;
 	if(editing_to_expression && current_from_struct && current_from_struct->isDateTime()) to_type = 3;
 	if(current_object_start < 0) {
-		if(editing_to_expression && current_from_struct && !current_from_units.empty()) {
+		if(editing_to_expression && editing_to_expression1 && current_from_struct && !current_from_units.empty()) {
 			to_type = 4;
 		} else if(editing_to_expression && editing_to_expression1 && current_from_struct) {
 			to_type = 2;
@@ -887,7 +952,7 @@ void do_completion(bool to_menu) {
 		gchar *gstr2 = gtk_text_buffer_get_text(expression_edit_buffer(), &object_start, &object_end, FALSE);
 		str = gstr2;
 		g_free(gstr2);
-		if(str.length() < (size_t) completion_min) {gtk_widget_hide(completion_window); return;}
+		if(unicode_length(str) < (size_t) completion_min) {gtk_widget_hide(completion_window); return;}
 	}
 	GtkTreeIter iter;
 	int matches = 0;
@@ -919,7 +984,7 @@ void do_completion(bool to_menu) {
 		if(current_function && current_function->subtype() == SUBTYPE_DATA_SET) {
 			arg = current_function->getArgumentDefinition(current_function_index);
 			if(arg && (arg->type() == ARGUMENT_TYPE_DATA_OBJECT || arg->type() == ARGUMENT_TYPE_DATA_PROPERTY)) {
-				if(arg->type() == ARGUMENT_TYPE_DATA_OBJECT && (str.empty() || str.length() < (size_t) completion_min)) {gtk_widget_hide(completion_window); return;}
+				if(arg->type() == ARGUMENT_TYPE_DATA_OBJECT && (str.empty() || unicode_length(str) < (size_t) completion_min)) {gtk_widget_hide(completion_window); return;}
 				if(current_function_index == 1) {
 					for(size_t i = 1; i <= current_function->countNames(); i++) {
 						if(str.find(current_function->getName(i).name) != string::npos) {
@@ -1026,13 +1091,13 @@ void do_completion(bool to_menu) {
 		if(!arg) {
 			vector<string> pstr;
 			vector<Prefix*> prefixes;
-			if(str.length() > (size_t) completion_min) {
+			if(!str.empty()) {
 				for(size_t pi = 1; ; pi++) {
 					Prefix *prefix = CALCULATOR->getPrefix(pi);
 					if(!prefix) break;
 					for(size_t name_i = 1; name_i <= prefix->countNames(); name_i++) {
 						const string *pname = &prefix->getName(name_i).name;
-						if(!pname->empty() && pname->length() < str.length() - completion_min + 1) {
+						if(!pname->empty() && pname->length() < str.length()) {
 							bool pmatch = true;
 							for(size_t i = 0; i < pname->length(); i++) {
 								if((*pname)[i] != str[i]) {
@@ -1048,8 +1113,6 @@ void do_completion(bool to_menu) {
 					}
 				}
 			}
-			GtkTreeIter exact_prefix_match;
-			bool exact_match_found = false, exact_prefix_match_found = false;
 			do {
 				ExpressionItem *item = NULL;
 				Prefix *prefix = NULL;
@@ -1103,7 +1166,7 @@ void do_completion(bool to_menu) {
 									b_match = (ename->name == str) ? 1 : 0;
 								} else {
 									for(size_t icap = 0; icap < 2; icap++) {
-										const std::string *namestr;
+										const string *namestr;
 										if(icap == 0) {
 											namestr = &ename->name;
 										} else {
@@ -1131,11 +1194,17 @@ void do_completion(bool to_menu) {
 												if(b_match && (!cu || (exp == 1 && cu->countUnits() == 1)) && ((!ename->case_sensitive && equalsIgnoreCase(*namestr, *cmpstr)) || (ename->case_sensitive && *namestr == *cmpstr))) b_match = 1;
 												if(b_match) {
 													if(icmp > 0 && !cu) {
-														if(CALCULATOR->getActiveVariable(str.substr(0, namestr->length() + (str.length() - cmpstr->length()))) || CALCULATOR->getActiveFunction(str.substr(0, namestr->length() + (str.length() - cmpstr->length())))) {
+														prefix = prefixes[icmp - 1];
+														if((prefix->type() != PREFIX_DECIMAL || ((Unit*) item)->minPreferredPrefix() > ((DecimalPrefix*) prefix)->exponent() || ((Unit*) item)->maxPreferredPrefix() < ((DecimalPrefix*) prefix)->exponent()) && (prefix->type() != PREFIX_BINARY || ((Unit*) item)->baseUnit()->referenceName() != "bit" || ((Unit*) item)->minPreferredPrefix() > ((BinaryPrefix*) prefix)->exponent() || ((Unit*) item)->maxPreferredPrefix() < ((BinaryPrefix*) prefix)->exponent())) {
 															b_match = false;
+															prefix = NULL;
+															continue;
+														}
+														if(CALCULATOR->getActiveExpressionItem(str.substr(0, str.length() - cmpstr->length()) + *namestr)) {
+															b_match = false;
+															prefix = NULL;
 															continue;
 														}
-														prefix = prefixes[icmp - 1];
 														i_match = str.length() - cmpstr->length();
 													} else if(b_match > 1 && !editing_to_expression && item->isHidden() && str.length() == 1) {
 														b_match = 4;
@@ -1164,8 +1233,12 @@ void do_completion(bool to_menu) {
 									cap_it = capitalized_names.find(&item->getName(name_i));
 									if(item->getName(name_i).name == *cmpstr || (cap_it != capitalized_names.end() && cap_it->second == *cmpstr)) {
 										if(!cu) {
-											if(icmp > 0) prefix = prefixes[icmp - 1];
-											else prefix = NULL;
+											if(icmp > 0) {
+												if(CALCULATOR->getActiveExpressionItem(str.substr(0, str.length() - cmpstr->length()) + *cmpstr)) break;
+												prefix = prefixes[icmp - 1];
+											} else {
+												prefix = NULL;
+											}
 										}
 										b_match = 1; break;
 									}
@@ -1214,14 +1287,6 @@ void do_completion(bool to_menu) {
 							}
 						}
 						if(gstr) g_free(gstr);
-						if(b_match == 1 && item->type() != TYPE_FUNCTION) {
-							if(prefix) {
-								exact_prefix_match = iter;
-								exact_prefix_match_found = true;
-							} else {
-								exact_match_found = true;
-							}
-						}
 						if(b_match > highest_match) highest_match = b_match;
 					}
 				} else if(item && to_type == 4) {
@@ -1237,6 +1302,7 @@ void do_completion(bool to_menu) {
 								if(b_match == 6) break;
 							}
 						}
+						if(b_match == 6 && item->isHidden() && item != CALCULATOR->getLocalCurrency()) b_match = 0;
 						if(b_match == 6) {
 							gchar *gstr;
 							gtk_tree_model_get(GTK_TREE_MODEL(completion_store), &iter, 0, &gstr, -1);
@@ -1282,7 +1348,7 @@ void do_completion(bool to_menu) {
 						if(current_from_struct && str.length() < 3) {
 							if(p_type >= 100 && p_type < 200) {
 								if(to_type == 5 || current_from_struct->containsType(STRUCT_UNIT) <= 0) b_match = 0;
-							} else if((p_type == 294 || (p_type == 292 && to_type == 4)) && !current_from_units.empty()) {
+							} else if((p_type == 294 || p_type == 295 || (p_type == 292 && to_type == 4)) && !current_from_units.empty()) {
 								bool b = false;
 								for(size_t i = 0; i < current_from_units.size(); i++) {
 									if(current_from_units[i] == CALCULATOR->getDegUnit()) {
@@ -1290,12 +1356,36 @@ void do_completion(bool to_menu) {
 										break;
 									}
 								}
-								if(!b) b_match = 0;
+								if(!b) {
+									b_match = 0;
+								} else if(str.empty()) {
+									int base = to_base;
+									if(base == 0) base = printops.base;
+									if((base == BASE_SEXAGESIMAL && p_type == 292) || (base == BASE_TIME && p_type == 293) || (base == BASE_LATITUDE && p_type == 294) || (base == BASE_LONGITUDE && p_type == 295)) {
+										b_match = 0;
+									}
+								}
 							} else if(p_type > 290 && p_type < 300 && (p_type != 292 || to_type >= 1)) {
 								if(!current_from_struct->isNumber() || (p_type > 290 && str.empty() && current_from_struct->isInteger())) b_match = 0;
+								if(str.empty()) {
+									int base = to_base;
+									if(base == 0) base = printops.base;
+									if((base == BASE_SEXAGESIMAL && p_type == 292) || (base == BASE_TIME && p_type == 293) || (base == BASE_LATITUDE && p_type == 294) || (base == BASE_LONGITUDE && p_type == 295)) {
+										b_match = 0;
+									}
+								}
 							} else if(p_type >= 200 && p_type <= 290 && (p_type != 200 || to_type == 1 || to_type >= 3)) {
-								if(!current_from_struct->isNumber()) b_match = 0;
-								else if(str.empty() && p_type >= 202 && !current_from_struct->isInteger()) b_match = 0;
+								if(!current_from_struct->isNumber()) {
+									b_match = 0;
+								} else if(str.empty() && p_type >= 202 && !current_from_struct->isInteger()) {
+									b_match = 0;
+								} else if(str.empty()) {
+									int base = to_base;
+									if(base == 0) base = printops.base;
+									if((p_type >= 202 && p_type <= 236 && base == p_type - 200) || (base == BASE_UNICODE && p_type == 281) || (base == BASE_BINARY_DECIMAL && p_type == 285) || (base == BASE_BIJECTIVE_26 && p_type == 290) || (base == BASE_ROMAN_NUMERALS && p_type == 280)) {
+										b_match = 0;
+									}
+								}
 							} else if(p_type >= 300 && p_type < 400) {
 								if(p_type == 300) {
 									if(!contains_rational_number(to_menu && current_displayed_result() ? *current_displayed_result() : *current_from_struct)) b_match = 0;
@@ -1314,15 +1404,57 @@ void do_completion(bool to_menu) {
 									}
 								} else {
 									if(!current_from_struct->isNumber()) b_match = 0;
+									if(p_type >= 310 && p_type <= 314) {
+										int base = to_base;
+										if(base == 0) base = printops.base;
+										if((base == BASE_FP16 && p_type == 310) || (base == BASE_FP32 && p_type == 311) || (base == BASE_FP64 && p_type == 312) || (base == BASE_FP80 && p_type == 313) || (base == BASE_FP128 && p_type == 314)) {
+											b_match = 0;
+										}
+									}
 								}
 							} else if(p_type >= 400 && p_type < 500) {
 								if(!contains_imaginary_number(*current_from_struct)) b_match = 0;
+								if(str.empty()) {
+									if(p_type == 404 && evalops.complex_number_form == COMPLEX_NUMBER_FORM_RECTANGULAR) b_match = 0;
+									else if(p_type == 403 && evalops.complex_number_form == COMPLEX_NUMBER_FORM_POLAR) b_match = 0;
+									else if(p_type == 402 && evalops.complex_number_form == COMPLEX_NUMBER_FORM_EXPONENTIAL) b_match = 0;
+									else if(p_type == 401 && evalops.complex_number_form == COMPLEX_NUMBER_FORM_CIS && !complex_angle_form) b_match = 0;
+									else if(p_type == 400 && evalops.complex_number_form == COMPLEX_NUMBER_FORM_CIS && complex_angle_form) b_match = 0;
+								}
 							} else if(p_type >= 500 && p_type < 600) {
 								if(!current_from_struct->isDateTime()) b_match = 0;
 							} else if(p_type == 600) {
-								if(!current_from_struct->isInteger() && current_from_struct->containsType(STRUCT_ADDITION) <= 0) b_match = 0;
+								if(!current_from_units.empty() || (!current_from_struct->isInteger() && current_from_struct->containsType(STRUCT_ADDITION) <= 0)) b_match = 0;
 							} else if(p_type == 601) {
-								if(current_from_struct->containsType(STRUCT_ADDITION) <= 0) b_match = 0;
+								if(current_from_units.empty() || current_from_struct->containsType(STRUCT_ADDITION) <= 0) b_match = 0;
+							} else if(p_type >= 700 && p_type < 800) {
+								int exp = to_form;
+								if(exp == TO_FORM_OFF) exp = printops.min_exp;
+								if((p_type == 703 && (exp == 0 || !current_from_struct->isNumber())) || (p_type == 702 && exp == -3) || (p_type == 701 && exp > 0)) {
+									b_match = 0;
+								} else {
+									const Number *nr_p = from_struct_get_number(*current_from_struct);
+									if(!nr_p) {
+										b_match = 0;
+									} else {
+										int absexp = (exp < -1 ? -exp : exp);
+										Number nr(nr_p->hasImaginaryPart() ? nr_p->realPart() : *nr_p);
+										if(nr.isFraction()) {
+											nr.recip();
+											if(absexp == -1) absexp = PRECISION;
+										} else if(absexp == -1) {
+											absexp = PRECISION + 3;
+										}
+										nr.abs();
+										if(nr > Number(1, 1, 100)) {
+											if(p_type == 703 || (p_type == 701 && absexp <= 100 && exp > 0)) b_match = 0;
+										} else {
+											if((p_type == 703 && (absexp > 100 || nr < Number(1, 1, absexp))) || (p_type == 701 && absexp <= 100 && ((absexp > 0 && exp >= -1 && nr >= Number(1, 1, absexp)) || nr < Number(1, 1, 5))) || (p_type == 702 && nr < Number(1, 1, 5))) {
+												b_match = 0;
+											}
+										}
+									}
+								}
 							}
 						}
 						if(b_match > highest_match) highest_match = b_match;
@@ -1336,10 +1468,6 @@ void do_completion(bool to_menu) {
 					else if(b_match < 3) show_separator1 = true;
 				}
 			} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(completion_store), &iter));
-			if(exact_match_found && exact_prefix_match_found) {
-				gtk_list_store_set(completion_store, &exact_prefix_match, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, -1);
-				matches--;
-			}
 		}
 	}
 	if(to_menu) current_from_struct = NULL;
@@ -1635,6 +1763,11 @@ void update_completion() {
 				else gtk_list_store_set(completion_store, &iter, 0, ename_r->name.c_str(), 1, CALCULATOR->variables[i]->title().c_str(), 2, CALCULATOR->variables[i], 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 1, 9, NULL, -1);
 				if(CALCULATOR->variables[i] == CALCULATOR->v_percent) gtk_list_store_set(completion_store, &iter, 9, (CALCULATOR->variables[i]->title() + " (%)").c_str(), -1);
 				else if(CALCULATOR->variables[i] == CALCULATOR->v_permille) gtk_list_store_set(completion_store, &iter, 9, (CALCULATOR->variables[i]->title() + " (‰)").c_str(), -1);
+				if(printops.use_unicode_signs && CALCULATOR->variables[i]->title(false).find("MeV/c^2") != string::npos) {
+					string title = CALCULATOR->variables[i]->title();
+					gsub("MeV/c^2", "MeV/c²", title);
+					gtk_list_store_set(completion_store, &iter, 1, title.c_str(), -1);
+				}
 			} else {
 				Variable *v = CALCULATOR->variables[i];
 				string title;
@@ -1645,7 +1778,7 @@ void update_completion() {
 						title = localize_expression(((KnownVariable*) v)->expression());
 						if(unicode_length(title) > 30) {
 							size_t n = 30;
-							while(n > 0 && (signed char) title[n + 1] < 0 && (unsigned char) title[n + 1] < 0xC0) n--;
+							while(n > 0 && (signed char) title[n] < 0 && (unsigned char) title[n] < 0xC0) n--;
 							title = title.substr(0, n); title += "…";
 						} else if(!((KnownVariable*) v)->unit().empty() && ((KnownVariable*) v)->unit() != "auto") {
 							title += " "; title += ((KnownVariable*) v)->unit();
@@ -1666,7 +1799,7 @@ void update_completion() {
 							title = CALCULATOR->print(((KnownVariable*) v)->get(), 30, po);
 							if(unicode_length(title) > 30) {
 								size_t n = 30;
-								while(n > 0 && (signed char) title[n + 1] < 0 && (unsigned char) title[n + 1] < 0xC0) n--;
+								while(n > 0 && (signed char) title[n] < 0 && (unsigned char) title[n] < 0xC0) n--;
 								title = title.substr(0, n);
 								title += "…";
 							}
@@ -1710,7 +1843,11 @@ void update_completion() {
 	for(size_t i = 0; i < CALCULATOR->units.size(); i++) {
 		Unit *u = CALCULATOR->units[i];
 		if(u->isActive()) {
-			if(u->subtype() != SUBTYPE_COMPOSITE_UNIT) {
+			CompositeUnit *cu = (u->subtype() == SUBTYPE_COMPOSITE_UNIT ? (CompositeUnit*) u : NULL);
+			if(!cu && u->isHidden() && u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT && !((AliasUnit*) u)->firstBaseUnit()->isHidden() && ((AliasUnit*) u)->firstBaseUnit()->isActive()) {
+				cu = (CompositeUnit*) ((AliasUnit*) u)->firstBaseUnit();
+			}
+			if(!cu) {
 				gtk_list_store_append(completion_store, &iter);
 				const ExpressionName *ename, *ename_r;
 				bool b = false;
@@ -1767,8 +1904,7 @@ void update_completion() {
 				}
 				if(b) gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, title.c_str(), 2, u, 3, FALSE, 4, 0, 5, it_flag == flag_surfaces.end() ? NULL : it_flag->second, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 1, 9, title2.empty() ? NULL : title2.c_str(), -1);
 				else gtk_list_store_set(completion_store, &iter, 0, ename_r->name.c_str(), 1, title.c_str(), 2, u, 3, FALSE, 4, 0, 5, it_flag == flag_surfaces.end() ? NULL : it_flag->second, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 1, 9, title2.empty() ? NULL : title2.c_str(), -1);
-			} else if(!u->isHidden()) {
-				CompositeUnit *cu = (CompositeUnit*) u;
+			} else if(!cu->isHidden()) {
 				Prefix *prefix = NULL;
 				int exp = 1;
 				title = cu->title(true, printops.use_unicode_signs, &can_display_unicode_string_function, (void*) completion_view);
@@ -1776,7 +1912,8 @@ void update_completion() {
 				bool tp = title2[title2.length() - 1] == ')';
 				title2 += " ";
 				if(!tp) title2 += "(";
-				if(cu->countUnits() == 1 && (u = cu->get(1, &exp, &prefix)) != NULL && prefix != NULL && exp == 1) {
+				Unit *u_i = NULL;
+				if(cu->countUnits() == 1 && (u_i = cu->get(1, &exp, &prefix)) != NULL && prefix != NULL && exp == 1) {
 					str = "";
 					for(size_t name_i = 0; name_i < 2; name_i++) {
 						const ExpressionName *ename;
@@ -1785,7 +1922,7 @@ void update_completion() {
 							bool b_italic = !str.empty();
 							if(b_italic) str += " <i>";
 							str += ename->formattedName(-1, false, true);
-							str += u->preferredInputName(name_i != 1, printops.use_unicode_signs, false, false, &can_display_unicode_string_function, (void*) expression_edit_widget()).formattedName(TYPE_UNIT, true, true);
+							str += u_i->preferredInputName(name_i != 1, printops.use_unicode_signs, false, false, &can_display_unicode_string_function, (void*) expression_edit_widget()).formattedName(TYPE_UNIT, true, true);
 							if(b_italic) str += "</i>";
 							if(!b_italic) title2 += str;
 						}
@@ -1813,7 +1950,7 @@ void update_completion() {
 						else title = cu->category().substr(i_slash, cu->category().length() - i_slash);
 					}
 				}
-				gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, title.c_str(), 2, cu, 3, FALSE, 4, 0, 5, NULL, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 1, 9, title2.c_str(), -1);
+				gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, title.c_str(), 2, u, 3, FALSE, 4, 0, 5, NULL, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 1, 9, title2.c_str(), -1);
 			}
 		}
 	}
@@ -1832,6 +1969,7 @@ void update_completion() {
 			if(ename && ename != ename_r && !ename->completion_only && !ename->plural && (!ename->unicode || can_display_unicode_string_function(ename->name.c_str(), (void*) expression_edit_widget()))) {
 				str += " <i>";
 				str += ename->formattedName(-1, false, true);
+				if(ename->name != "k" && ename->name != "M") str += "-";
 				str += "</i>";
 			}
 		}
@@ -1878,10 +2016,14 @@ void update_completion() {
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Calendars"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 500, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("cis")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Complex cis Form"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 401, 9, NULL, -1);
+	COMPLETION_CONVERT_STRING("decimals")
+	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Decimal Fraction"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 302, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("decimal") str += " <i>"; str += "dec"; str += "</i>";
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Decimal Number"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 210, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("duodecimal") str += " <i>"; str += "duo"; str += "</i>";
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Duodecimal Number"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 212, 9, NULL, -1);
+	COMPLETION_CONVERT_STRING("engineering") str += " <i>"; str += "eng"; str += "</i>";
+	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Engineering Notation"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 702, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("exponential")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Complex Exponential Form"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 402, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("factors")
@@ -1900,14 +2042,12 @@ void update_completion() {
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Fraction"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 300, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("1/")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, (string(_("Fraction")) + " (1/n)").c_str(), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 301, 9, NULL, -1);
-	COMPLETION_CONVERT_STRING("decimals")
-	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Decimal Fraction"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 302, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("hexadecimal") str += " <i>"; str += "hex"; str += "</i>";
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Hexadecimal Number"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 216, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("latitude") str += " <i>"; str += "latitude2"; str += "</i>";
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Latitude"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 294, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("longitude") str += " <i>"; str += "longitude2"; str += "</i>";
-	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Longitude"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 294, 9, NULL, -1);
+	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Longitude"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 295, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("mixed")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Mixed Units"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 102, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("octal") str += " <i>"; str += "oct"; str += "</i>";
@@ -1924,8 +2064,12 @@ void update_completion() {
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Complex Rectangular Form"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 404, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("roman")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Roman Numerals"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 280, 9, NULL, -1);
+	COMPLETION_CONVERT_STRING("scientific") str += " <i>"; str += "sci"; str += "</i>";
+	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Scientific Notation"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 701, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("sexagesimal") str += " <i>"; str += "sexa"; str += "</i>"; str += " <i>"; str += "sexa2"; str += "</i>"; str += " <i>"; str += "sexa3"; str += "</i>";
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Sexagesimal Number"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 292, 9, NULL, -1);
+	COMPLETION_CONVERT_STRING("simple")
+	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Simple Notation"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 703, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("time")
 	gtk_list_store_append(completion_store, &iter); gtk_list_store_set(completion_store, &iter, 0, str.c_str(), 1, _("Time Format"), 2, NULL, 3, FALSE, 4, 0, 6, PANGO_WEIGHT_NORMAL, 7, 0, 8, 293, 9, NULL, -1);
 	COMPLETION_CONVERT_STRING("unicode")
